/**
 * - Les méthodes commencant par "execute" calcule quelque chose, l'affiche et le retourne
 * - Les méthodes commencant par "refesh" affichent des messages
 */

var Calcul = {

    invalidClass: 'LV_invalid_field',

    /**
     * Récupère un nombre à partir d'un champ
     */
    get: function(field, round)
    {
        return Calcul.round(Number($F(field).replace(',','.')), round);
    },

    /**
     * Fixe un champ sur une valeur avec l'arrondi qui va bien
     */
    set: function(field, value, round)
    {
        var hasValue = value && value!=Infinity ? true : false;
        $(field).value = hasValue ? Calcul.round(value, round) : '';
    },

    round: function(num, round)
    {
        if(typeof round != "undefined")
        {
            var m = Math.pow(10, round);
            num = Math.round(num*m) / m;
        }
        return num;
    },

    executeIMC: function()
    {
        //10000 * poids / taille^2
        var imc = 10000 * Calcul.get('c_poids') / Math.pow(Calcul.get('c_taille'), 2);

        //Mise à jour champ
        Calcul.set('c_imc', imc, 2);

        //MAJ conclusion
        Calcul.refreshConclusionAndDiagnostique();

        return imc;
    },
    
    executeMDRD: function(event)
    {
        if(event && event.target.id == 'c_creatinine')
        {
            LiveValidation.massValidate([Calcul.v_sexe, Calcul.v_age, Calcul.v_poids, Calcul.v_taille]);
        }

        var sexeMultiplier = $('c_sexeF').checked ? 0.742 : 1;
        var afroMultiplier = $('c_afroY').checked ? 1.212 : 1;
        var creatinine     = Calcul.get('c_creatinine');
        var age            = Calcul.get('c_age');
        var r = 186 * Math.pow(creatinine/88.9, -1.154) * Math.pow(age, -0.203) * sexeMultiplier * afroMultiplier;
        
        Calcul.set('c_mdrd', r, 1);

        //MAJ conclusion
        Calcul.refreshConclusionAndDiagnostique();

        return r;
    },

    executeInvalidationUrine24: function()
    {
        var c24 = Calcul.get('c_creatininurie24');
        var cAt = Calcul.get('c_creatinineAttendue');
        var invalidation = ! (!c24 || c24 < 0.6*cAt || c24 > 1.4*cAt);

        $('c_invalidationUrine24').value = invalidation ? 'true':'false';

        //Blocage des zones nécéssitant des données correctes
        if( ! invalidation)
        {
            $$('.requires-valid-urine input').each(function(e){
               e.writeAttribute('disabled', true);
               e.value = '';
            });
        }
        else
        {
            $$('.requires-valid-urine input').invoke('writeAttribute', 'disabled', false);
        }

        //Messages
        if(!c24)
        {
            $('messageOnglet2').update(langue.get('messageOnglet2_1'));
        }
        else if( ! invalidation)
        {
            $('messageOnglet2').update(langue.get('messageOnglet2_2'));
        }
        else
        {
            $('messageOnglet2').update('');
        }

        //Rafraichissement des zones
        Calcul.refreshHabitudeAlimentaire();
        Calcul.refreshInstructionRemplissage();

        return invalidation;
    },

    executeCreatinineAttendue: function()
    {
        var r;
        if($('c_sexeM').checked || $('c_sexeF').checked)
        {
            var multiplier = $('c_sexeM').checked ? 1.77 : 1.5;
            r = (140 - Calcul.get('c_age')) * Calcul.get('c_poids') * multiplier / 1000;
        }
        else
        {
            r = '';
        }
        
        Calcul.set('c_creatinineAttendue', r, 3);

        //Affichage du message
        if(r && Calcul.get('c_poids'))
        {
            var creatAttMol = Calcul.round(r, 1);
            var creatAttMg  = Calcul.round(r / 0.00885, 2);
            $('infoParametre').update(sprintf(langue.get('infoParametre'), creatAttMol, creatAttMg));
        }
        else
        {
            $('infoParametre').update('');
        }

        //On rafraichit l'invalidation
        Calcul.executeInvalidationUrine24();
        
        return r;
    },

    refreshHabitudeAlimentaire: function()
    {
        var string = [];
        
        var diurese24 = Calcul.get('c_diurese24');
        if(diurese24 && diurese24 < 1500) string.push(langue.get('infoHabitudeMessage1'));
        else if(diurese24 >= 1500 && diurese24 < 2000) string.push(langue.get('infoHabitudeMessage2'));
        else if(diurese24 >= 2000) string.push(langue.get('infoHabitudeMessage3'));

        var na24 = Calcul.get('c_na24');
        if(na24)
        {
            string.push(sprintf(langue.get('infoHabitudeMessage4'), Calcul.round(na24/17, 1)));
        }

        var uree24  = Calcul.get('c_uree24');
        var poids   = Calcul.get('c_poids');
        if(uree24 && poids)
        {
            string.push(sprintf(langue.get('infoHabitudeMessage5'), Calcul.round(0.21*uree24/poids, 1)));
        }
        

        $('infoHabitude').update(string.join('<br/>'));
    },

    refreshConclusionAndDiagnostique: function()
    {
        var string = [];

        //reset
        $('c_calcium24').removeClassName(Calcul.invalidClass);
        $('c_calcium24G').removeClassName(Calcul.invalidClass);
        $('c_acideurique24').removeClassName(Calcul.invalidClass);
        $('c_acideurique24G').removeClassName(Calcul.invalidClass);
        $('c_uree24').removeClassName(Calcul.invalidClass);
        $('c_uree24G').removeClassName(Calcul.invalidClass);

        //variables diverses
        var poids       = Calcul.get('c_poids');
        var diurese24   = Calcul.get('c_diurese24');

        //Conclusion IMC
        var imc = Calcul.get('c_imc', 1);
        if( ! imc)          {/*rien*/}
        else if(imc < 16.5) string.push(sprintf(langue.get('conclusionIMC1'), imc));
        else if(imc < 18.5) string.push(sprintf(langue.get('conclusionIMC2'), imc));
        else if(imc < 25)   string.push(sprintf(langue.get('conclusionIMC3'), imc));
        else if(imc < 30)   string.push(sprintf(langue.get('conclusionIMC4'), imc));
        else if(imc < 35)   string.push(sprintf(langue.get('conclusionIMC5'), imc));
        else if(imc < 40)   string.push(sprintf(langue.get('conclusionIMC6'), imc));
        else                string.push(sprintf(langue.get('conclusionIMC7'), imc));

        //Conclusion MDRD
        var mdrd = Calcul.get('c_mdrd');
        if( ! mdrd)        {/*rien*/}
        else if(mdrd < 15) string.push(langue.get('conclusionMDRD1'));
        else if(mdrd < 30) string.push(langue.get('conclusionMDRD2'));
        else if(mdrd < 60) string.push(langue.get('conclusionMDRD3'));
        else if(mdrd < 90) string.push(langue.get('conclusionMDRD4'));
        else                string.push(langue.get('conclusionMDRD5'));

        //Calcémie
        var calcium = Calcul.get('c_calcium');
        if( ! calcium)         {/*rien*/}
        else if(calcium > 2.5) string.push(langue.get('conclusionCalcemie1'));
        else if(calcium < 2.2) string.push(langue.get('conclusionCalcemie2'));

        //Uricémie
        var acideurique = Calcul.get('c_acideurique');
        if(acideurique && acideurique > 380) string.push(langue.get('conclusionUricemie1'));

        //Glycémie
        var glycemie = Calcul.get('c_glycemie');
        if(glycemie && glycemie >= 7) string.push(langue.get('conclusionGlycemie1'));

        //Calciurie
        var calcium24 = Calcul.get('c_calcium24');
        if( ! calcium24)
        {
            /*rien*/
        }
        else if(calcium24 > poids*0.1)
        {
            $('c_calcium24').addClassName(Calcul.invalidClass);
            $('c_calcium24G').addClassName(Calcul.invalidClass);
            string.push(langue.get('conclusionCalciurie1'));
        }
        else if(diurese24 && calcium24/diurese24*1000 > 3.8 )
        {
            string.push(langue.get('conclusionCalciurie2'));
        }

        //Uraturie
        var acideurique24 = Calcul.get('c_acideurique24');
        if( ! acideurique24)
        {
            /*rien*/
        }
        else if(acideurique24 > poids*0.07)
        {
            $('c_acideurique24').addClassName(Calcul.invalidClass);
            $('c_acideurique24G').addClassName(Calcul.invalidClass);
            string.push(langue.get('conclusionUraturie1'));
        }
        else if(diurese24 && acideurique24/diurese24*1000 > 2.4 )
        {
            string.push(langue.get('conclusionUraturie2'));
        }

        //Glycémie
        var uree24 = Calcul.get('c_uree24');
        if(uree24 && uree24/poids > 5.7)
        {
            $('c_uree24').addClassName(Calcul.invalidClass);
            $('c_uree24G').addClassName(Calcul.invalidClass);
            string.push(langue.get('conclusionUree'));
        }

        //Natriurese
        var na24 = Calcul.get('c_na24');
        if(na24 && na24/17 > 9) string.push(langue.get('conclusionNa'));

        //Natriurese
        var densiteReveil = Calcul.get('c_densitereveil');
        if(densiteReveil && densiteReveil > 1020) string.push(langue.get('conclusionDensiteReveil'));

        $('infoConclusion').update(string.join('<br/>'));

        /** DIAGNOSTIQUE **/
        string = [];

        //Age
        var age = Calcul.get('c_age');
        if(age && age < 20) string.push(langue.get('diagMessageAge'));

        //Diurese
        if(diurese24 && diurese24<2000) string.push(langue.get('diagMessageDiurese1'));
        if(diurese24 && densiteReveil && diurese24>=2000 && densiteReveil>=1020) string.push(langue.get('diagMessageDiurese2'));

        //Calcium
        if(calcium && calcium>2.5) string.push(langue.get('diagMessageCalcium'));

        //MDRD
        if(mdrd && mdrd<60) string.push(langue.get('diagMessageMDRD'));

        //Na
        if(na24 && calcium24 && na24/17>9 && calcium24>poids*0.1) string.push(langue.get('diagMessageNa'));

        //Uree
        if(uree24 && calcium24 && uree24*0.21/poids>1.2 && calcium24>poids*0.1) string.push(langue.get('diagMessageUree'));

        //AcideUrique
        var phReveil = Calcul.get('c_phreveil');
        if(acideurique && phReveil && imc && acideurique>380 && imc>25 && phReveil<=5.5) string.push(langue.get('diagMessageAcideUrique'));

        //phReveil
        if(phReveil && phReveil>7) string.push(langue.get('diagMessagePh'));

        $('infoDiagnostique').update(string.join('<br/>'));
    },

    refreshInstructionRemplissage: function()
    {
        var string      = [];
        var calcium24   = Calcul.get('c_calcium24');
        var creati24    = Calcul.get('c_creatininurie24');
        var creatiAt    = Calcul.get('c_creatinineAttendue');
        var poids       = Calcul.get('c_poids');
        var age         = Calcul.get('c_age');
        var sexe        = $F('c_sexeValidator');
        var taille      = Calcul.get('c_taille');

        $('c_creatininurie24').removeClassName(Calcul.invalidClass);
        $('c_creatininurie24G').removeClassName(Calcul.invalidClass);

        if(Calcul.v_sexe && (poids || sexe ||taille || age))
        {
            LiveValidation.massValidate([Calcul.v_sexe, Calcul.v_age, Calcul.v_poids, Calcul.v_taille]);
        }
        
        if( creati24 && poids && sexe && taille)
        {
            if(creati24 < 0.6*creatiAt)
            {
                $('c_creatininurie24').addClassName(Calcul.invalidClass);
                $('c_creatininurie24G').addClassName(Calcul.invalidClass);
                string.push(langue.get('infoRemplissageAutre1'));
            }
            else if (creati24 < 0.7*creatiAt)
            {
                string.push(langue.get('infoRemplissageAutre5')); //<-- nouveau message
            }
            else if(creati24 < 1.3*creatiAt)
            {
                string.push(langue.get('infoRemplissageAutre2'));
            }
            else if(creati24 < 1.4*creatiAt)
            {
                string.push(langue.get('infoRemplissageAutre5')); //<-- nouveau message
            }
            else if(creati24 >= 1.4*creatiAt)
            {
                $('c_creatininurie24').addClassName(Calcul.invalidClass);
                $('c_creatininurie24G').addClassName(Calcul.invalidClass);
                string.push(langue.get('infoRemplissageAutre3'));
            }
        }

        if(calcium24 && calcium24<=poids*0.1)
        {
            string.push(langue.get('infoRemplissageAutre4'));
        }

        $('infoRemplissageAutre').update(string.join('<br/>'));
    },

    refreshInvalidationUrineReveil: function()
    {
        var message = '';
        if($('ecbu_n').checked || $('ecbu_p').checked)
        {
            if($('ecbu_p').checked)
            {
                message = langue.get('infoReveilMessage2');
            }
        }
        else
        {
            message = langue.get('infoReveilMessage1');
        }

        $('messageOnglet3').update(message);
    },

    setupValidation: function()
    {
        /************** VALIDATIONS **************/
        //Validation Remplissage
        Calcul.v_sexe          = new LiveValidation( 'c_sexeValidator', {
            insertAfterWhatNode: $('infoRemplissageSexe')
        }).add(Validate.Presence, {
            failureMessage: langue.get('infoRemplissageSexe')
        });
        Calcul.v_age           = new LiveValidation( 'c_age', {
            insertAfterWhatNode: $('infoRemplissageAge')
        }).add(Validate.Presence, {
            failureMessage: langue.get('infoRemplissageAge')
        });
        Calcul.v_poids         = new LiveValidation( 'c_poids', {
            insertAfterWhatNode: $('infoRemplissagePoids')
        }).add(Validate.Presence, {
            failureMessage: langue.get('infoRemplissagePoids')
        });
        Calcul.v_taille        = new LiveValidation( 'c_taille', {
            insertAfterWhatNode: $('infoRemplissageTaille')
        }).add(Validate.Presence, {
            failureMessage: langue.get('infoRemplissageTaille')
        });

        //Validation Onglet 1
        Calcul.v_calcium       = new LiveValidation( 'c_calcium').add(Validate.Numericality, {minimum: 2.2, maximum: 2.5});
        Calcul.v_acideurique   = new LiveValidation( 'c_acideurique').add(Validate.Numericality, {maximum: 380});
        Calcul.v_glycemie      = new LiveValidation( 'c_glycemie').add(Validate.Numericality, {maximum: 7.001});

        //Validations Onglet 2
        Calcul.v_na24          = new LiveValidation( 'c_na24').add(Validate.Numericality, {maximum: 150});

        //Validations Onglet 3
        Calcul.v_densitereveil = new LiveValidation( 'c_densitereveil').add(Validate.Numericality, {maximum: 1020.000001});
        
        /************** RELATIONS VALIDATIONS **************/
        //Liens de rafraichissement validation Onglet 1
        $('c_calciumG').observe('blur', Calcul.v_calcium.validate.bind(Calcul.v_calcium));
        $('c_acideuriqueG').observe('blur', Calcul.v_acideurique.validate.bind(Calcul.v_acideurique));
        $('c_glycemieG').observe('blur', Calcul.v_glycemie.validate.bind(Calcul.v_glycemie));

        //Liens de rafraichissement validation Onglet 2
//        $('c_acideurique24G').observe('blur', Calcul.v_acideurique24.validate.bind(Calcul.v_acideurique24));
//        $('c_calcium24G').observe('blur', Calcul.v_calcium24.validate.bind(Calcul.v_calcium24));
//        $('c_uree24G').observe('blur', Calcul.v_uree24.validate.bind(Calcul.v_uree24));

        //Initialisation des calculs
        Calcul.initialize();
    },

    initialize: function()
    {
        Calcul.executeCreatinineAttendue();
        Calcul.executeIMC();
        Calcul.executeInvalidationUrine24();
        Calcul.executeMDRD();
        Calcul.refreshConclusionAndDiagnostique();
        Calcul.refreshHabitudeAlimentaire();
        Calcul.refreshInstructionRemplissage();
        Calcul.refreshInvalidationUrineReveil();
    },

    reset: function()
    {
        $$('input').each(function(e){
            switch(e.readAttribute('type').toLowerCase())
            {
                case 'checkbox':
                case 'radio':
                    e.checked = false;
                break;

                default:
                    e.value = '';
                break;
            }
        })

        Calcul.initialize();
    }
}